@@ -219,7 +219,7 @@ bool aiounicast_nonblock::Send
 	mpz_t tmp;
 	mpz_init_set(tmp, m);
 	if (aio_is_encrypted)
-		mpz_add(tmp, tmp, aio_hide_length);
+		mpz_add(tmp, tmp, aio_hide_length); // add $2^c$ to hide length
 	size_t size = mpz_sizeinbase(tmp, TMCG_MPZ_IO_BASE);
 	if ((size * 2) >= buf_in_size)
 	{
@@ -241,7 +241,7 @@ bool aiounicast_nonblock::Send
 	}
 	else
 	{
-		std::cerr << "aiounicast_nonblock: realsize does not fit" << std::endl;
+		std::cerr << "aiounicast_nonblock(" << j << "): realsize does not fit" << std::endl;
 		delete [] buf;
 		return false;
 	}
@@ -256,7 +256,7 @@ bool aiounicast_nonblock::Send
 	if (aio_is_encrypted)
 	{
 		memmove(buf + 1, buf, realsize - 1);
-		buf[0] = '+'; // set plus-character als non-zero prefix
+		buf[0] = '+'; // use plus-character as a non-zero prefix
 		err = gcry_cipher_encrypt(*enc_out[i_in], buf + 1, realsize - 1, NULL, 0);
 		if (err)
 		{
@@ -285,7 +285,7 @@ bool aiounicast_nonblock::Send
 		}
 		else
 		{
-			std::cerr << "aiounicast_nonblock: realsize does not fit" << std::endl;
+			std::cerr << "aiounicast_nonblock(" << j << "): realsize does not fit" << std::endl;
 			delete [] buf;
 			return false;
 		}
@@ -329,7 +329,7 @@ bool aiounicast_nonblock::Send
 				iv_flag_out[i_in] = true; // IV has been sent
 		}
 	}
-	// calculate the MAC over all data including line delimiter
+	// calculate the MAC over all data including the line delimiter
 	if (aio_is_authenticated)
 	{
 		err = gcry_mac_write(*mac_out[i_in], buf, realsize);
