@@ -3,7 +3,7 @@
 
    This file is part of LibTMCG.
 
- Copyright (C) 2004, 2005, 2007, 2016, 2017  Heiko Stamer <HeikoStamer@gmx.net>
+ Copyright (C) 2004, 2005, 2007, 2016--2018  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,8 +49,20 @@ bool init_libTMCG
 			TMCG_LIBGCRYPT_VERSION << " needed" << std::endl;
 		return false;
 	}
-	gcry_control(GCRYCTL_DISABLE_SECMEM, 0); // disable secure memory
-	gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+	if (!gcry_control(GCRYCTL_INITIALIZATION_FINISHED_P))
+	{
+		gcry_control(GCRYCTL_DISABLE_SECMEM, 0); // disable secure memory
+		gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+	}
+	// check libgcrypt
+	gcry_error_t ret = gcry_control(GCRYCTL_SELFTEST);
+	if (ret)
+	{
+		std::cerr << "init_libTMCG(): libgcrypt self-test " <<
+			"failed (rc = " << gcry_err_code(ret) << ", str = " <<
+			gcry_strerror(ret) << ")" << std::endl;
+		return false;
+	}
 	if (gcry_md_test_algo(TMCG_GCRY_MD_ALGO)) // check for digest algorithm
 	{
 		std::cerr << "init_libTMCG(): libgcrypt algorithm " <<
