@@ -94,25 +94,44 @@ void mpz_grandomb
 	(mpz_ptr r, const unsigned long int size, enum gcry_random_level level)
 {
 	unsigned char *rtmp;
-	char htmp[size + 3]; // at least two characters + delimiter
+	char htmp[size + 3]; /* at least two characters + delimiter */
 	size_t hlen;
 	gcry_mpi_t rr;
+	gcry_error_t ret;
 	assert(size <= UINT_MAX);
 	
 	rr = gcry_mpi_new((unsigned int)size);
 	gcry_mpi_randomize(rr, (unsigned int)size, level);
-	gcry_mpi_aprint(GCRYMPI_FMT_HEX, &rtmp, &hlen, rr);
+	ret = gcry_mpi_aprint(GCRYMPI_FMT_HEX, &rtmp, &hlen, rr);
+	if (ret)
+	{
+		fprintf(stderr, "mpz_grandomb(): gcry_mpi_aprint() failed: %s\n", gcry_strerror(ret));
+		mpz_set_ui(r, 0L); /* indicates an error */
+	}
+	else
+	{
+		memset(htmp, 0, size + 3);
+		memcpy(htmp, rtmp, hlen);
+		gcry_free(rtmp);
+		mpz_set_str(r, htmp, 16);
+		mpz_tdiv_r_2exp(r, r, size); /* r mod 2^size, i.e. shift right and bit mask */
+	}
 	gcry_mpi_release(rr);
-	memset(htmp, 0, size + 3);
-	memcpy(htmp, rtmp, hlen);
-	gcry_free(rtmp);
-	mpz_set_str(r, htmp, 16);
-	mpz_tdiv_r_2exp(r, r, size); /* r mod 2^size, i.e. shift right and bit mask */
 }
 
 void mpz_ssrandomb
 	(mpz_ptr r, const unsigned long int size)
 {
+	FILE *fhd = fopen("/proc/sys/kernel/random/entropy_avail", "r");
+	if (fhd != NULL)
+	{
+		unsigned long int entropy_avail = 0;
+		if (fscanf(fhd, "%lu", &entropy_avail) != 1)
+			entropy_avail = 0;
+		fclose(fhd);
+		if (entropy_avail < size)
+			fprintf(stderr, "mpz_ssrandomb(): too few entropy (%lu bits) available; blocking\n", entropy_avail);
+	}
 	mpz_grandomb(r, size, GCRY_VERY_STRONG_RANDOM);
 }
 
@@ -131,27 +150,46 @@ void mpz_wrandomb
 void mpz_grandomm
 	(mpz_ptr r, mpz_srcptr m, enum gcry_random_level level)
 {
-	unsigned long int size = mpz_sizeinbase(m, 2L);
+	unsigned long int size = mpz_sizeinbase(m, 2L) + 64; /* cf. BSI TR-02102-1, B.4 Verfahren 2 */
 	unsigned char *rtmp;
-	char htmp[size + 3]; // at least two characters + delimiter
+	char htmp[size + 3]; /* at least two characters + delimiter */
 	size_t hlen;
 	gcry_mpi_t rr;
+	gcry_error_t ret;
 	assert(size <= UINT_MAX);
 	
 	rr = gcry_mpi_new((unsigned int)size);
 	gcry_mpi_randomize(rr, (unsigned int)size, level);
-	gcry_mpi_aprint(GCRYMPI_FMT_HEX, &rtmp, &hlen, rr);
+	ret = gcry_mpi_aprint(GCRYMPI_FMT_HEX, &rtmp, &hlen, rr);
+	if (ret)
+	{
+		fprintf(stderr, "mpz_grandomm(): gcry_mpi_aprint() failed: %s\n", gcry_strerror(ret));
+		mpz_set_ui(r, 0L); /* indicates an error */
+	}
+	else
+	{
+		memset(htmp, 0, size + 3);
+		memcpy(htmp, rtmp, hlen);
+		gcry_free(rtmp);
+		mpz_set_str(r, htmp, 16);
+		mpz_mod(r, r, m); /* modulo bias is negligible here */
+	}
 	gcry_mpi_release(rr);
-	memset(htmp, 0, size + 3);
-	memcpy(htmp, rtmp, hlen);
-	gcry_free(rtmp);
-	mpz_set_str(r, htmp, 16);
-	mpz_mod(r, r, m); /* modulo bias is negligible here */
 }
 
 void mpz_ssrandomm
 	(mpz_ptr r, mpz_srcptr m)
 {
+	FILE *fhd = fopen("/proc/sys/kernel/random/entropy_avail", "r");
+	if (fhd != NULL)
+	{
+		unsigned long int entropy_avail = 0;
+		if (fscanf(fhd, "%lu", &entropy_avail) != 1)
+			entropy_avail = 0;
+		fclose(fhd);
+		if (entropy_avail < mpz_sizeinbase(m, 2L))
+			fprintf(stderr, "mpz_ssrandomm(): too few entropy (%lu bits) available; blocking\n", entropy_avail);
+	}
 	mpz_grandomm(r, m, GCRY_VERY_STRONG_RANDOM);
 }
 
@@ -166,3 +204,48 @@ void mpz_wrandomm
 {
 	mpz_grandomm(r, m, GCRY_WEAK_RANDOM);
 }
+
+void mpz_ssrandomm_cache_init
+	(mpz_t ssrandomm_cache[TMCG_MAX_SSRANDOMM_CACHE],
+	mpz_ptr ssrandomm_cache_mod,
+	size_t *ssrandomm_cache_avail,
+	size_t n, mpz_srcptr m)
+{
+	size_t i = 0;
+	if ((n == 0) || (n > TMCG_MAX_SSRANDOMM_CACHE))
+		return;
+	for (i = 0; i < TMCG_MAX_SSRANDOMM_CACHE; i++)
+		mpz_init(ssrandomm_cache[i]);	
+	for (i = 0; i < n; i++)
+		mpz_ssrandomm(ssrandomm_cache[i], m);
+	mpz_init_set(ssrandomm_cache_mod, m);
+	*ssrandomm_cache_avail = n;
+}
+
+void mpz_ssrandomm_cache
+	(mpz_t ssrandomm_cache[TMCG_MAX_SSRANDOMM_CACHE],
+	mpz_srcptr ssrandomm_cache_mod,
+	size_t *ssrandomm_cache_avail,
+	mpz_ptr r, mpz_srcptr m)
+{
+	if (!mpz_cmp(m, ssrandomm_cache_mod) && *ssrandomm_cache_avail)
+	{
+		(*ssrandomm_cache_avail)--; // next cached random value
+		mpz_set(r, ssrandomm_cache[*ssrandomm_cache_avail]);
+	}
+	else
+		mpz_ssrandomm(r, m);
+}
+
+void mpz_ssrandomm_cache_done
+	(mpz_t ssrandomm_cache[TMCG_MAX_SSRANDOMM_CACHE],
+	mpz_ptr ssrandomm_cache_mod,
+	size_t *ssrandomm_cache_avail)
+{
+	size_t i = 0;
+	*ssrandomm_cache_avail = 0;
+	mpz_clear(ssrandomm_cache_mod);
+	for (i = 0; i < TMCG_MAX_SSRANDOMM_CACHE; i++)
+		mpz_clear(ssrandomm_cache[i]);
+}
+
